# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawBillingClient, RawBillingClient
from .types.get_billing_response import GetBillingResponse


class BillingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBillingClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBillingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBillingClient
        """
        return self._raw_client

    def get_billing_information_for_the_organization(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBillingResponse:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBillingResponse
            Returns billing information including storage usage and ledger data

        Examples
        --------
        from ittybit import Ittybit
        client = Ittybit(token="YOUR_TOKEN", )
        client.billing.get_billing_information_for_the_organization()
        """
        _response = self._raw_client.get_billing_information_for_the_organization(request_options=request_options)
        return _response.data


class AsyncBillingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBillingClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBillingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBillingClient
        """
        return self._raw_client

    async def get_billing_information_for_the_organization(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBillingResponse:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBillingResponse
            Returns billing information including storage usage and ledger data

        Examples
        --------
        from ittybit import AsyncIttybit
        import asyncio
        client = AsyncIttybit(token="YOUR_TOKEN", )
        async def main() -> None:
            await client.billing.get_billing_information_for_the_organization()
        asyncio.run(main())
        """
        _response = await self._raw_client.get_billing_information_for_the_organization(request_options=request_options)
        return _response.data
